@@ -1,5 +1,5 @@
 /* This file is part of the Zebra server.
-   Copyright (C) 1994-2010 Index Data
+   Copyright (C) 1994-2011 Index Data
 
 Zebra is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -17,6 +17,9 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 */
 
+#if HAVE_CONFIG_H
+#include <config.h>
+#endif
 #include <assert.h>
 #include <string.h>
 #include <stdlib.h>
@@ -25,18 +28,15 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 #include <yaz/proto.h>
 #include <idzebra/data1.h>
 
-static int *f_integer(data1_node *c, ODR o)
+static Odr_int *f_integer(data1_node *c, ODR o)
 {
-    int *r;
     char intbuf[64];
 
     if (!c->child || c->child->which != DATA1N_data ||
 	c->child->u.data.len > 63)
 	return 0;
-    r = (int *)odr_malloc(o, sizeof(*r));
     sprintf(intbuf, "%.*s", 63, c->child->u.data.data);
-    *r = atoi(intbuf);
-    return r;
+    return odr_intdup(o, atoi(intbuf));
 }
 
 static char *f_string(data1_node *c, ODR o)
