@@ -45,11 +45,11 @@ const char *myrec[] = {
         "<gils>\n<title>My computer</title>\n</gils>\n",
         "<gils>\n<title>My x computer</title>\n</gils>\n",
         "<gils>\n<title>My computer x</title>\n</gils>\n" ,
-        "<gils>\n<title>" char_ae "</title>\n</gils>\n" ,
+        "<gils>\n<title>" char_ae "rme</title>\n</gils>\n" ,
         "<gils>\n<title>B" char_aring "d</title>\n"
         "<abstract>זיהוי סדר הארועים בסיפור המרד הגדול מאת צביה בן-שלום 提示:直接点击数据库名称,将进入单库检索 Ngày xửa ngày xưa D.W. all wet</abstract>\n</gils>\n" ,
 	0} ;
-	
+
 static void tst(int argc, char **argv)
 {
 #if YAZ_HAVE_ICU
@@ -62,10 +62,59 @@ static void tst(int argc, char **argv)
 
     /* simple term */
     YAZ_CHECK(tl_query(zh, "@attr 1=title notfound", 0));
- 
+
     YAZ_CHECK(tl_query(zh, "@attr 1=title computer", 3));
 
-    YAZ_CHECK(tl_query(zh, "@attr 1=title @attr 5=1 comput", 3));
+    YAZ_CHECK(tl_query(zh, "@attr 5=1 @attr 1=title computer", 3));
+
+    YAZ_CHECK(tl_query(zh, "@attr 5=1 @attr 1=title compute", 3));
+
+    YAZ_CHECK(tl_query(zh, "@attr 5=1 @attr 1=title computee", 0));
+
+    YAZ_CHECK(tl_query(zh, "@attr 5=1 @attr 1=title co", 3));
+
+    YAZ_CHECK(tl_query(zh, "@attr 5=2 @attr 1=title computer", 3));
+
+    YAZ_CHECK(tl_query(zh, "@attr 5=2 @attr 1=title compute", 0));
+
+    YAZ_CHECK(tl_query(zh, "@attr 5=2 @attr 1=title er", 3));
+
+    YAZ_CHECK(tl_query(zh, "@attr 5=3 @attr 1=title computer", 3));
+
+    YAZ_CHECK(tl_query(zh, "@attr 5=3 @attr 1=title compute", 3));
+
+    YAZ_CHECK(tl_query(zh, "@attr 5=3 @attr 1=title er", 4));
+
+    YAZ_CHECK(tl_query(zh, "@attr 5=3 @attr 1=title ompute", 3));
+
+    YAZ_CHECK(tl_query(zh, "@attr 5=102 @attr 1=title com.*er", 3));
+
+    YAZ_CHECK(tl_query(zh, "@attr 5=102 @attr 1=title cm.*er", 0));
+
+    YAZ_CHECK(tl_query(zh, "@attr 5=102 @attr 1=title com.*ër", 3));
+
+    YAZ_CHECK(tl_query(zh, "@attr 5=102 @attr 1=title com?m.*er", 3));
+
+    YAZ_CHECK(tl_query(zh, "@attr 5=102 @attr 1=title coy?m.*er", 3));
+
+    YAZ_CHECK(tl_query(zh, "@attr 5=102 @attr 1=title co[m].*er", 3));
+
+    YAZ_CHECK(tl_query(zh, "@attr 5=102 @attr 1=title co[mn].*er", 3));
+
+    YAZ_CHECK(tl_query(zh, "@attr 5=102 @attr 1=title co[m-n].*er", 3));
+
+#if 0
+    /* fails on some systems with older ICU */
+    YAZ_CHECK(tl_query(zh, "@attr 5=102 @attr 1=title co[a-z].*er", 3));
+
+    YAZ_CHECK(tl_query(zh, "@attr 5=102 @attr 1=title co[a-n].*er", 3));
+#endif
+
+    YAZ_CHECK(tl_query(zh, "@attr 1=title com.*ër", 0));
+
+    YAZ_CHECK(tl_query(zh, "@attr 1=title @and @attr 5=102 com.*er x", 2));
+
+    YAZ_CHECK(tl_query(zh, "@attr 1=title @and x @attr 5=102 com.*er", 2));
 
     YAZ_CHECK(tl_query(zh, "@attr 1=title .computer.", 3));
 
@@ -75,14 +124,20 @@ static void tst(int argc, char **argv)
 
     YAZ_CHECK(tl_query(zh, "@attr 1=title mY", 3));
 
-    YAZ_CHECK(tl_query(zh, char_ae, 1));
-    YAZ_CHECK(tl_query(zh, char_AE, 1));
+    YAZ_CHECK(tl_query(zh, char_ae "rme", 1));
+    YAZ_CHECK(tl_query(zh, char_AE "RME", 1));
 
     YAZ_CHECK(tl_query(zh, "b" char_aring "d", 1));
     YAZ_CHECK(tl_query(zh, "B" char_Aring "D", 1));
     YAZ_CHECK(tl_query(zh, "b" char_aring1 "d", 1));
     YAZ_CHECK(tl_query(zh, "B" char_Aring1 "D", 1));
 
+    YAZ_CHECK(tl_query(zh, "@attr 5=102 b" char_aring "d", 1));
+    YAZ_CHECK(tl_query(zh, "@attr 5=102 b.d", 1));
+
+    YAZ_CHECK(tl_query(zh, "@attr 5=102 " char_ae "rme", 1));
+    YAZ_CHECK(tl_query(zh, "@attr 5=102 " "..rme", 1));
+
     /* Abstract searches . Chinese mostly */
     YAZ_CHECK(tl_query(zh, "@attr 1=abstract בן", 1));
     YAZ_CHECK(tl_query(zh, "@attr 1=abstract צביה", 1));
@@ -98,6 +153,8 @@ static void tst(int argc, char **argv)
     YAZ_CHECK(tl_query(zh, "@attr 1=abstract @attr 5=1 בס", 1));
     YAZ_CHECK(tl_query(zh, "@attr 1=abstract @attr 5=1 ב", 1));
 
+    YAZ_CHECK(tl_query(zh, "@attr 1=abstract @attr 5=102 בן", 1));
+
     /* phrase search */
     YAZ_CHECK(tl_query(zh, "@attr 1=title {my computer}", 2));
     YAZ_CHECK(tl_query(zh, "@attr 1=title @attr 6=1 {my computer}", 2));
@@ -112,10 +169,10 @@ static void tst(int argc, char **argv)
     YAZ_CHECK(tl_query(zh, "@attr 1=title @attr 2=103 {}", 5));
     YAZ_CHECK(tl_query(zh, "@attr 1=abstract @attr 2=103 {}", 1));
     YAZ_CHECK(tl_query(zh, "@attr 1=abstract @attr 2=103 {does not match}", 1));
-    
+
     /* scan */
     {   /* word search */
-        const char *ent[] = { char_ae, "B" char_aring "d", "computer", 
+        const char *ent[] = { char_ae "rme", "B" char_aring "d", "computer",
                               "My", "x", 0 };
         YAZ_CHECK(tl_scan(zh, "@attr 1=title 0", 1, 10, 1, 5, 1, ent));
     }
@@ -126,10 +183,10 @@ static void tst(int argc, char **argv)
     }
 
     {   /* phrase search */
-        const char *ent[] = { char_ae, "B" char_aring "d", "My computer" };
+        const char *ent[] = { char_ae "rme", "B" char_aring "d", "My computer" };
         YAZ_CHECK(tl_scan(zh, "@attr 1=title @attr 6=2 0", 1, 3, 1, 3, 0, ent));
     }
-    
+
     YAZ_CHECK(tl_close_down(zh, zs));
 #endif
 }
